"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_debounce_1 = __importDefault(require("lodash.debounce"));
var react_1 = require("react");
/**
 * @typeparam T - The type of items to search.
 */
var Searchable = /** @class */ (function (_super) {
    __extends(Searchable, _super);
    function Searchable(props) {
        var _this = _super.call(this, props) || this;
        /**
         * Change event handler for updating query in state.
         *
         * @param event - An instance of ChangeEvent
         */
        _this.handleChange = function (event) {
            return _this.setState({ query: event.target.value });
        };
        var query = props.initialQuery, predicate = props.predicate, items = props.items, debounce = props.debounce, filter = props.filter;
        _this.state = {
            items: query !== ''
                ? Searchable.filter(items, query, predicate)
                : filter
                    ? items
                    : [],
            query: query,
        };
        if (debounce) {
            var duration = typeof debounce === 'number'
                ? debounce
                : Searchable.defaultProps.debounce;
            _this.filterAndSetState = lodash_debounce_1.default(_this.filterAndSetState, duration);
        }
        return _this;
    }
    /**
     * Filters an array of items based on a search query and a filtering predicate.
     *
     * @typeparam T - The type of items.
     * @param items - The array of items to be filtered.
     * @param query - A search string to be passed to the predicate.
     * @param predicate - A filtering predicate based on an item and the search query.
     * @returns A filtered array of items.
     */
    Searchable.filter = function (items, query, predicate) {
        return items.filter(function (item) { return predicate(item, query); });
    };
    /**
     * Call filtering method if [[IState.query]] has changed.
     */
    Searchable.prototype.componentDidUpdate = function (prevProps, prevState) {
        var query = this.state.query;
        var items = this.props.items;
        if (query !== prevState.query || items !== prevProps.items) {
            this.filterAndSetState(items, query);
        }
    };
    Searchable.prototype.render = function () {
        var _a = this.state, items = _a.items, query = _a.query;
        var renderParams = { items: items, query: query, handleChange: this.handleChange };
        if (isPropsWithRender(this.props)) {
            return this.props.render(renderParams);
        }
        else {
            return this.props.children(renderParams);
        }
    };
    /**
     * Filters an array of items and sets state. This method might be debounced in [[constructor]].
     *
     * @param items - The array of items to filter.
     * @param query - The search query to base the filtering on.
     */
    Searchable.prototype.filterAndSetState = function (items, query) {
        var _a = this.props, predicate = _a.predicate, filter = _a.filter;
        this.setState({
            items: query !== ''
                ? Searchable.filter(items, query, predicate)
                : filter
                    ? items
                    : [],
        });
    };
    Searchable.defaultProps = {
        debounce: 300,
        filter: false,
        initialQuery: '',
    };
    return Searchable;
}(react_1.Component));
exports.default = Searchable;
function isPropsWithRender(props) {
    return props.render !== undefined;
}
